using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using RepPP;
using System.Xml;
using RepPP.Toolkit.Window;

namespace RepPP.Toolkit.Window {

    //*********************************************************
    //
    /// <summary>
    /// Class handling New MDI form. There must be one instance of this class
    /// for each MDI list of top form. Each top form must register to the list
    /// using the RegisterForm method.
    /// </summary>
    //
    //*********************************************************
    public class NMDIFormHelperUI : NMDIFormHelperBase {
        private const int                           m_iSep3IDX = 9;


        //******************************************************************
        //
        /// <summary>
        /// ctor:                           Class constructor.
        /// </summary>
        //
        //******************************************************************
        public NMDIFormHelperUI() {
        }

        //******************************************************************
        //
        /// <summary>
        /// SetMenuText:                Set the text of a menu item
        /// </summary>
        /// <param name="trans">        Translator</param>
        /// <param name="strName">      Tag name</param>
        /// <param name="mi">           Menu item</param>
        //
        //******************************************************************
#if VC8
        private void SetMenuText(ControlTextTranslator trans, ToolStripMenuItem mi, string strName) {
            string  strText;
            
            strText = trans.GetItemText(strName, "Text");
            if (strText != null && strText != String.Empty) {
                mi.Text = strText;
            }
        }
#endif

        //******************************************************************
        //
        /// <summary>
        /// SetMenuText:                Set the text of a menu item
        /// </summary>
        /// <param name="trans">        Translator</param>
        /// <param name="strName">      Tag name</param>
        /// <param name="mi">           Menu item</param>
        //
        //******************************************************************
        private void SetMenuText(ControlTextTranslator trans, MenuItem mi, string strName) {
            string  strText;
            
            strText = trans.GetItemText(strName, "Text");
            if (strText != null && strText != String.Empty) {
                mi.Text = strText;
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// FillWindowsMenu:            Initialize the windows menu
        /// </summary>
        /// <param name="cmdHolder">    Command holder</param>
        /// <param name="miWindows">    Windows menu item</param>
        /// <param name="trans">        Translator</param>
        //
        //******************************************************************
        protected virtual void FillWindowsMenu(CmdHolder cmdHolder, MenuItem miWindows, ControlTextTranslator trans) {
            MenuItem       miMinimizeAll;
            MenuItem       miRestoreAll;
            MenuItem       miMaximizeAll;
            MenuItem       miArrangeVert;
            MenuItem       miArrangeHorz;
            MenuItem       miArrangeCasc;
            MenuItem       miCloseAll;
            MenuItem       miSep1;
            MenuItem       miSep2;
            MenuItem       miSep3;
            
            miMinimizeAll       = new MenuItem();
            miMinimizeAll.Index = 0;
            miMinimizeAll.Text  = "&Minimize All";
            miRestoreAll        = new MenuItem();
            miRestoreAll.Index  = 1;
            miRestoreAll.Text   = "&Restore All";
            miMaximizeAll       = new MenuItem();
            miMaximizeAll.Index = 2;
            miMaximizeAll.Text  = "Ma&ximize All";
            miSep1              = new MenuItem();
            miSep1.Index        = 3;
            miSep1.Text         = "-";
            miArrangeVert       = new MenuItem();
            miArrangeVert.Index = 4;
            miArrangeVert.Text  = "Arrange &Vertically";
            miArrangeHorz       = new MenuItem();
            miArrangeHorz.Index = 5;
            miArrangeHorz.Text  = "Arrange &Horizontally";
            miArrangeCasc       = new MenuItem();
            miArrangeCasc.Index = 6;
            miArrangeCasc.Text  = "&Cascade";
            miSep2              = new MenuItem();
            miSep2.Index        = 7;
            miSep2.Text         = "-";
            miCloseAll          = new MenuItem();
            miCloseAll.Index    = 8;
            miCloseAll.Text     = "Close &All";
            miSep3              = new MenuItem();
            miSep3.Index        = 9;
            miSep3.Text         = "-";
            if (trans != null) {
                SetMenuText(trans, miMinimizeAll, "miWindowsMinimizeAll");
                SetMenuText(trans, miRestoreAll,  "miWindowsRestoreAll");
                SetMenuText(trans, miMaximizeAll, "miWindowsMaximizeAll");
                SetMenuText(trans, miArrangeVert, "miWindowsArrangeVertically");
                SetMenuText(trans, miArrangeHorz, "miWindowsArrangeHorizontally");
                SetMenuText(trans, miArrangeCasc, "miWindowsArrangeCascade");
                SetMenuText(trans, miCloseAll,    "miWindowsCloseAll");
            }
            miWindows.MenuItems.AddRange(new MenuItem[] {
                                                            miMinimizeAll,
                                                            miRestoreAll,
                                                            miMaximizeAll,
                                                            miSep1,
                                                            miArrangeVert,
                                                            miArrangeHorz,
                                                            miArrangeCasc,
                                                            miSep2,
                                                            miCloseAll,
                                                            miSep3});
            cmdHolder.AddTrigger(null, "WindowsMinimizeAll",         new object[] { miMinimizeAll });
            cmdHolder.AddTrigger(null, "WindowsMaximizeAll",         new object[] { miMaximizeAll });
            cmdHolder.AddTrigger(null, "WindowsRestoreAll",          new object[] { miRestoreAll });
            cmdHolder.AddTrigger(null, "WindowsArrangeVertically",   new object[] { miArrangeVert });
            cmdHolder.AddTrigger(null, "WindowsArrangeHorizontally", new object[] { miArrangeHorz });
            cmdHolder.AddTrigger(null, "WindowsArrangeInCascade",    new object[] { miArrangeCasc });
            cmdHolder.AddTrigger(null, "WindowsCloseAll",            new object[] { miCloseAll });
            miWindows.Popup += new System.EventHandler(OnWindowsMenuPopup);
        }

        //******************************************************************
        //
        /// <summary>
        /// FillWindowsMenu:            Initialize the windows menu
        /// </summary>
        /// <param name="cmdHolder">    Command holder</param>
        /// <param name="miWindows">    Windows menu item</param>
        /// <param name="trans">        Translator</param>
        //
        //******************************************************************
#if VC8
        protected virtual void FillWindowsMenu(CmdHolder cmdHolder, ToolStripMenuItem miWindows, ControlTextTranslator trans) {
            ToolStripMenuItem   miMinimizeAll;
            ToolStripMenuItem   miRestoreAll;
            ToolStripMenuItem   miMaximizeAll;
            ToolStripMenuItem   miArrangeVert;
            ToolStripMenuItem   miArrangeHorz;
            ToolStripMenuItem   miArrangeCasc;
            ToolStripMenuItem   miCloseAll;
            ToolStripSeparator  miSep1;
            ToolStripSeparator  miSep2;
            ToolStripSeparator  miSep3;
            
            miMinimizeAll       = new ToolStripMenuItem();
            miMinimizeAll.Text  = "&Minimize All";
            miRestoreAll        = new ToolStripMenuItem();
            miRestoreAll.Text   = "&Restore All";
            miMaximizeAll       = new ToolStripMenuItem();
            miMaximizeAll.Text  = "Ma&ximize All";
            miSep1              = new ToolStripSeparator();
            miSep1.Text         = "-";
            miArrangeVert       = new ToolStripMenuItem();
            miArrangeVert.Text  = "Arrange &Vertically";
            miArrangeHorz       = new ToolStripMenuItem();
            miArrangeHorz.Text  = "Arrange &Horizontally";
            miArrangeCasc       = new ToolStripMenuItem();
            miArrangeCasc.Text  = "&Cascade";
            miSep2              = new ToolStripSeparator();
            miSep2.Text         = "-";
            miCloseAll          = new ToolStripMenuItem();
            miCloseAll.Text     = "Close &All";
            miSep3              = new ToolStripSeparator();
            miSep3.Text         = "-";
            if (trans != null) {
                SetMenuText(trans, miMinimizeAll, "miWindowsMinimizeAll");
                SetMenuText(trans, miRestoreAll,  "miWindowsRestoreAll");
                SetMenuText(trans, miMaximizeAll, "miWindowsMaximizeAll");
                SetMenuText(trans, miArrangeVert, "miWindowsArrangeVertically");
                SetMenuText(trans, miArrangeHorz, "miWindowsArrangeHorizontally");
                SetMenuText(trans, miArrangeCasc, "miWindowsArrangeCascade");
                SetMenuText(trans, miCloseAll,    "miWindowsCloseAll");
            }
            miWindows.DropDownItems.AddRange(new ToolStripItem[] {
                                                            miMinimizeAll,
                                                            miRestoreAll,
                                                            miMaximizeAll,
                                                            miSep1,
                                                            miArrangeVert,
                                                            miArrangeHorz,
                                                            miArrangeCasc,
                                                            miSep2,
                                                            miCloseAll,
                                                            miSep3});
            cmdHolder.AddTrigger(null, "WindowsMinimizeAll",         new object[] { miMinimizeAll });
            cmdHolder.AddTrigger(null, "WindowsMaximizeAll",         new object[] { miMaximizeAll });
            cmdHolder.AddTrigger(null, "WindowsRestoreAll",          new object[] { miRestoreAll });
            cmdHolder.AddTrigger(null, "WindowsArrangeVertically",   new object[] { miArrangeVert });
            cmdHolder.AddTrigger(null, "WindowsArrangeHorizontally", new object[] { miArrangeHorz });
            cmdHolder.AddTrigger(null, "WindowsArrangeInCascade",    new object[] { miArrangeCasc });
            cmdHolder.AddTrigger(null, "WindowsCloseAll",            new object[] { miCloseAll });
            miWindows.DropDownOpening += new EventHandler(OnDropDownOpening);
        }

#endif

        //******************************************************************        
        //
        /// <summary>
        /// RegisterForm:                   Register a form
        /// </summary>
        /// <param name="frm">              Form to register</param>
        /// <param name="bVisible">         true to show form in the top form list</param>
        /// <param name="miWindow">         Windows menu to be filled</param>
        /// <param name="cmdHolder">        Command holder</param>
        /// <param name="trans">            Control text translator use to set the menu text</param>
        //
        //******************************************************************        
        public virtual void RegisterForm(Form frm, bool bVisible, MenuItem miWindow, CmdHolder cmdHolder, ControlTextTranslator trans) {
            RegisterForm(frm, bVisible, cmdHolder);
            FillWindowsMenu(cmdHolder, miWindow, trans);
        }

        //******************************************************************        
        //
        /// <summary>
        /// RegisterForm:                   Register a form
        /// </summary>
        /// <param name="frm">              Form to register</param>
        /// <param name="bVisible">         true to show form in the top form list</param>
        /// <param name="miWindow">         Windows menu to be filled</param>
        /// <param name="cmdHolder">        Command holder</param>
        /// <param name="trans">            Control text translator use to set the menu text</param>
        //
        //******************************************************************        
#if VC8
        public virtual void RegisterForm(Form frm, bool bVisible, ToolStripMenuItem miWindow, CmdHolder cmdHolder, ControlTextTranslator trans) {
            RegisterForm(frm, bVisible, cmdHolder);
            FillWindowsMenu(cmdHolder, miWindow, trans);
        }
#endif

        //******************************************************************
        //
        /// <summary>
        /// RefreshWindowsMenu:		    Adjust Windows menu depending on the current context.
        /// </summary>
        /// <param name="mi">           Menu item to adjust</param>
        /// <param name="frmActive">    Currently active top form</param>
        /// <param name="cmdHolder">    Command holder</param>
        //
        //******************************************************************
        protected virtual void RefreshWindowsMenu(MenuItem mi, Form frmActive, CmdHolder cmdHolder) {
            int         iCount = 1;
            string		strText;
            MenuItem    miItem;
            MenuItem    miSep3;

            RefreshWindowsCmd(frmActive, cmdHolder);
            while (mi.MenuItems.Count > (m_iSep3IDX + 1)) {
                mi.MenuItems.RemoveAt(m_iSep3IDX + 1);
            }
            miSep3 = mi.MenuItems[m_iSep3IDX];
            if (m_iVisibleCount != 0) {
                miSep3.Visible = true;
                foreach (TopForm topFormChild in m_arrTopForm) {
                    if (topFormChild.bVisible) {
                        strText				    = iCount.ToString() + " " + topFormChild.frmTop.Text;
                        miItem				    = new MenuItem("&" + strText) ;
                        miItem.Click	       += new EventHandler(OnNMDIFormSelect);
                        if (topFormChild.frmTop == frmActive) {
                            miItem.Checked = true;
                        }
                        mi.MenuItems.Add(miItem);
                        iCount++;
                    }
                }
            } else {
                miSep3.Visible = false;
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// RefreshWindowsMenu:		    Adjust Windows menu depending on the current context.
        /// </summary>
        /// <param name="mi">           Menu item to adjust</param>
        /// <param name="frmActive">    Currently active top form</param>
        /// <param name="cmdHolder">    Command holder</param>
        //
        //******************************************************************
#if VC8
        protected virtual void RefreshWindowsMenu(ToolStripMenuItem mi, Form frmActive, CmdHolder cmdHolder) {
            int                 iCount = 1;
            string		        strText;
            ToolStripMenuItem   miItem;
            ToolStripSeparator  miSep3;

            RefreshWindowsCmd(frmActive, cmdHolder);
            while (mi.DropDownItems.Count > (m_iSep3IDX + 1)) {
                mi.DropDownItems.RemoveAt(m_iSep3IDX + 1);
            }
            miSep3 = mi.DropDownItems[m_iSep3IDX] as ToolStripSeparator;
            if (m_iVisibleCount != 0) {
                miSep3.Visible = true;
                foreach (TopForm topFormChild in m_arrTopForm) {
                    if (topFormChild.bVisible) {
                        strText				    = iCount.ToString() + " " + topFormChild.frmTop.Text;
                        miItem				    = new ToolStripMenuItem("&" + strText) ;
                        miItem.Click	       += new EventHandler(OnNMDIFormSelect);
                        if (topFormChild.frmTop == frmActive) {
                            miItem.Checked = true;
                        }
                        mi.DropDownItems.Add(miItem);
                        iCount++;
                    }
                }
            } else {
                miSep3.Visible = false;
            }
        }
#endif

        //******************************************************************
        //
        /// <summary>
        /// OnWindowsMenuPopup:		Refresh the windows menu on popup
        /// </summary>
        /// <param name="sender">   Event sender</param>
        /// <param name="e">        Event arguments</param>
        //
        //******************************************************************
        private void OnWindowsMenuPopup(object sender, EventArgs e) {
            MenuItem    miWindows;
            
            if (m_topFormActive != null) {
                miWindows = (MenuItem)sender;
                RefreshWindowsMenu(miWindows, m_topFormActive.frmTop, m_topFormActive.cmdHolder);
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// OnDropDownOpening:		Refresh the windows menu on popup
        /// </summary>
        /// <param name="sender">   Event sender</param>
        /// <param name="e">        Event arguments</param>
        //
        //******************************************************************
#if VC8
        void OnDropDownOpening(object sender, EventArgs e) {
            ToolStripMenuItem   miWindows;
            
            if (m_topFormActive != null) {
                miWindows = (ToolStripMenuItem)sender;
                RefreshWindowsMenu(miWindows, m_topFormActive.frmTop, m_topFormActive.cmdHolder);
            }
        }
#endif

        //******************************************************************
        //
        /// <summary>
        /// OnNMDIFormSelect:       Activate the top form when menu is triggered
        /// </summary>
        /// <param name="sender">   Event sender</param>
        /// <param name="e">        Event arguments</param>
        //
        //******************************************************************
        protected virtual void OnNMDIFormSelect(object sender, EventArgs e) {
            int				iIndex;
		    
            if (m_arrTopForm != null) {
                if (sender is MenuItem) {
                    iIndex = ((MenuItem)sender).Index - m_iSep3IDX - 1;
#if VC8
                } else if (sender is ToolStripMenuItem) {
                    ToolStripMenuItem   mi = (ToolStripMenuItem)sender;
                    
                    iIndex = (mi.OwnerItem as ToolStripMenuItem).DropDownItems.IndexOf(mi) - m_iSep3IDX - 1;
#endif
                } else {
                    iIndex = -1;
                }
                foreach (TopForm topForm in m_arrTopForm) {
                    if (topForm.bVisible) {
                        if (iIndex == 0) {
                            topForm.frmTop.Activate();
                            break;
                        } else {
                            iIndex--;
                        }
                    }
                }
            }
        }
    } /* NMDIFormHelper */
} // Namespace
