using System;
using RepPP;

namespace RepPP.VSNetWizard
{
	/// <summary>
	/// ASPNetFieldGenerator: Generator of ASP.Net representation for rep++ field
	/// </summary>
	internal class ASPNetFieldGenerator
	{
        /// <summary>
        /// ASP.Net field control type enum
        /// </summary>
        public enum ASPNetFldCtlTypeE {
            sdFldUnknown                = 0,
            sdFldLabel                  = 1,
            sdFldTextBox                = 2,
            sdFldCheckBoxList           = 3,
            sdFldRadioButtonList        = 4,
            sdFldListBox                = 5,
            sdFldDropDownList           = 6,
            sdFldCalendar               = 7
        };
        /// <summary>Default memo control height</summary>
        public const int                DEF_MEMO_HEIGHT_PX          = 60;
        /// <summary>Default memo control width</summary>
        public const int                DEF_MEMO_WIDTH_PX           = 400;
        /// <summary>Default textbox height</summary>
        public const int                DEF_TEXTBOX_HEIGHT_PX       = 22;
        /// <summary>Default label height</summary>
        public const int                DEF_LABEL_HEIGHT_PX         = 22;
        /// <summary>Default dropdownlist height</summary>
        public const int                DEF_DROPDOWNLIST_HEIGHT_PX  = 22;
        /// <summary>Default checkbox item height</summary>
        public const int                DEF_CHECKBOX_HEIGHT_PX      = 22;
        /// <summary>Default radiobutton item height</summary>
        public const int                DEF_RADIOBUTTON_HEIGHT_PX   = 22;
        /// <summary>Default listbox height</summary>
        public const int                DEF_LISTBOX_HEIGHT_PX       = 100;
        /// <summary>Default calendar height</summary>
        public const int                DEF_CALENDAR_HEIGHT_PX      = 100;
        /// <summary>Default Rep++ control prefix</summary>
        public const string             DEF_CTL_PREFIX              = "reptk";
        /// <summary>Vertical margin bettewen controls</summary>
        public const int                CTL_VERTICAL_MARGIN         = 10;

        //********************************************************************************
        //
        /// <summary>
        /// ctor:                               Class constructor
        /// </summary>
        /// <param name="disp">                 Dispatch object</param>
        //
        //********************************************************************************
		public ASPNetFieldGenerator()
		{
		}

        //********************************************************************************
        //
        /// <summary>
        /// GetASPNetFldCtlType:    Get the rep++ ASP.Net control type for the field
        /// </summary>
        /// <param name="fld">      Rep++ field</param>
        /// <returns>
        /// ASP.Net control type
        /// </returns>
        //
        //********************************************************************************
        public static ASPNetFldCtlTypeE GetASPNetFldCtlType(RepPP.Field fld) {
            ASPNetFldCtlTypeE   eRetVal = ASPNetFldCtlTypeE.sdFldUnknown;
            RepPP.FieldType     eFldType;
            RepPP.CtlType       eDefCtlType;

            eFldType    = fld.Type;
            eDefCtlType = fld.ControlType;
            switch(eDefCtlType) {
            case RepPP.CtlType.sdCtlList:
                switch(eFldType) {
                case RepPP.FieldType.sdFieldSimpleAlphaChoice:
                case RepPP.FieldType.sdFieldSimpleNumChoice:
                    eRetVal = ASPNetFldCtlTypeE.sdFldDropDownList;
                    break;
                case RepPP.FieldType.sdFieldMultiAlphaChoice:
                case RepPP.FieldType.sdFieldMultiNumChoice:
                    eRetVal = ASPNetFldCtlTypeE.sdFldListBox;
                    break;
                default:
                    break;
                }
                break;
            case RepPP.CtlType.sdCtlCombo:
                eRetVal = ASPNetFldCtlTypeE.sdFldDropDownList;
                break;
            case RepPP.CtlType.sdCtlRadio:
                eRetVal = ASPNetFldCtlTypeE.sdFldRadioButtonList;
                break;
            case RepPP.CtlType.sdCtlCheck:
                eRetVal = ASPNetFldCtlTypeE.sdFldCheckBoxList;
                break;
            case RepPP.CtlType.sdCtlEdit:
            case RepPP.CtlType.sdCtlFmt:
            case RepPP.CtlType.sdCtlFmtDate:
                eRetVal = ASPNetFldCtlTypeE.sdFldTextBox;
                break;
            default:
                break;
            }
            if (eRetVal == ASPNetFldCtlTypeE.sdFldUnknown) {
                switch(eFldType) {
                case RepPP.FieldType.sdFieldSimpleAlphaChoice:
                case RepPP.FieldType.sdFieldSimpleNumChoice:
                    eRetVal = ASPNetFldCtlTypeE.sdFldDropDownList;
                    break;
                case RepPP.FieldType.sdFieldMultiAlphaChoice:
                case RepPP.FieldType.sdFieldMultiNumChoice:
                    eRetVal = ASPNetFldCtlTypeE.sdFldListBox;
                    break;
                case RepPP.FieldType.sdFieldMemo:
                    eRetVal = ASPNetFldCtlTypeE.sdFldTextBox;
                    break;
                default:
                    eRetVal = eRetVal = ASPNetFldCtlTypeE.sdFldTextBox;
                    break;
                }
            }
            return(eRetVal);
        }

        //********************************************************************************
        //
        /// <summary>
        /// GetASPNetFldCtlName:    Get the rep++ ASP.Net control name for the field
        /// </summary>
        /// <param name="fld">      Rep++ field control type</param>
        /// <returns>
        /// ASP.Net control type name
        /// </returns>
        //
        //********************************************************************************
        public static string GetASPNetFldCtlName(ASPNetFldCtlTypeE eCtlType) {
            string  strRetVal = null;

            switch (eCtlType) {
            case ASPNetFldCtlTypeE.sdFldLabel:
                strRetVal = "RepPP.Toolkit.Web.FldLabel";
                break;
            case ASPNetFldCtlTypeE.sdFldTextBox:
                strRetVal = "RepPP.Toolkit.Web.FldTextBox";
                break;
            case ASPNetFldCtlTypeE.sdFldDropDownList:
                strRetVal = "RepPP.Toolkit.Web.FldDropDownList";
                break;
            case ASPNetFldCtlTypeE.sdFldListBox:
                strRetVal = "RepPP.Toolkit.Web.FldListBox";
                break;
            case ASPNetFldCtlTypeE.sdFldRadioButtonList:
                strRetVal = "RepPP.Toolkit.Web.FldRadioButtonList";
                break;
            case ASPNetFldCtlTypeE.sdFldCheckBoxList:
                strRetVal = "RepPP.Toolkit.Web.FldCheckBoxList";
                break;
            case ASPNetFldCtlTypeE.sdFldCalendar:
                strRetVal = "RepPP.Toolkit.Web.FldCalendar";
                break;
            default:
                break;
            }
            return(strRetVal);
        }

        //********************************************************************************
        //
        /// <summary>
        /// GetFieldControlHeight:      Get the rep++ ASP.Net control height
        /// </summary>
        /// <param name="fld">          Rep++ field control type</param>
        /// <param name="eCtlType">     Control type</param>
        /// <returns>
        /// Height of the control
        /// </returns>
        //
        //********************************************************************************
        public static int GetFieldControlHeight(RepPP.Field  fld, ASPNetFldCtlTypeE eCtlType) {
            int iRetVal;

            switch (eCtlType) {
            case ASPNetFldCtlTypeE.sdFldLabel:
                iRetVal = DEF_LABEL_HEIGHT_PX;
                break;
            case ASPNetFldCtlTypeE.sdFldTextBox:
                if (fld.Type == FieldType.sdFieldMemo) {
                    iRetVal = DEF_MEMO_HEIGHT_PX;
                } else {
                    iRetVal = DEF_TEXTBOX_HEIGHT_PX;
                }
                break;
            case ASPNetFldCtlTypeE.sdFldDropDownList:
                iRetVal = DEF_DROPDOWNLIST_HEIGHT_PX;
                break;
            case ASPNetFldCtlTypeE.sdFldListBox:
                iRetVal = DEF_LISTBOX_HEIGHT_PX;
                break;
            case ASPNetFldCtlTypeE.sdFldRadioButtonList:
                iRetVal = DEF_RADIOBUTTON_HEIGHT_PX * fld.ChoiceList.Count;
                break;
            case ASPNetFldCtlTypeE.sdFldCheckBoxList:
                iRetVal = DEF_CHECKBOX_HEIGHT_PX * fld.ChoiceList.Count;
                break;
            case ASPNetFldCtlTypeE.sdFldCalendar:
                iRetVal = DEF_CALENDAR_HEIGHT_PX;
                break;
            default:
                iRetVal = -1;
                break;
            }
            return(iRetVal);
        }

        //********************************************************************************
        //
        /// <summary>
        /// BuildFieldControlBody:          build ASP.Net field control body text
        /// </summary>
        /// <param name="fld">              Rep++ field</param>
        /// <param name="strPrefixName">    Prefix name</param>
        /// <param name="iHeight">          Height</param>
        /// <param name="iWidth">           Width</param>
        /// <param name="iLeft">            Left position</param>
        /// <param name="iTop">             Top postion</param>
        /// <returns>
        /// ASP.Net control type
        /// </returns>
        //
        //********************************************************************************
        private static string BuildFieldControlBody(RepPP.Field          fld,
                                                    string               strPrefixName,
                                                    int                  iHeight,
                                                    int                  iWidth,
                                                    int                  iLeft,
                                                    int                  iTop) {
            string strRetVal;

            strRetVal = " id=\"" + strPrefixName + fld.Name + "\"";
            if (iLeft != -1 && iTop != -1) {
                strRetVal += " style=\"LEFT: " + iLeft + "px; POSITION: absolute; TOP: " + iTop + "px\"";
            }
            strRetVal += " runat=\"server\"" + 
                        " FieldFullName=\"" + fld.Group.Name + "." + fld.Name + "\"";
            if (iHeight != -1) {
                strRetVal += " Height=\"" + iHeight + "px\"";
            }
            if (iWidth != -1) {
               strRetVal += " Width=\"" + iWidth + "px\"";
            }
            return(strRetVal);
        }

        //********************************************************************************
        //
        /// <summary>
        /// GenASPNetFldCtlText:            Generate ASP.Net field control text
        /// </summary>
        /// <param name="fld">              Rep++ field</param>
        /// <param name="eCtlType">         ASP.Net control type</param>
        /// <param name="strCtlPrefix">     ASP.Net control prefix</param>
        /// <param name="iHeight">          Height</param>
        /// <param name="iWidth">           Width</param>
        /// <param name="iLeft">            Left position</param>
        /// <param name="iTop">             Top postion</param>
        /// <returns>
        /// ASP.Net control type
        /// </returns>
        //
        //********************************************************************************
        public static string GenASPNetFldCtlText(RepPP.Field          fld,
                                                 ASPNetFldCtlTypeE    eCtlType,
                                                 string               strCtlPrefix,
                                                 int                  iHeight,
                                                 int                  iWidth,
                                                 int                  iLeft,
                                                 int                  iTop) {
            string strRetVal = String.Empty;

            switch (eCtlType) {
            case ASPNetFldCtlTypeE.sdFldLabel:
                strRetVal = "<" + strCtlPrefix + ":FldLabel" +
                            BuildFieldControlBody(fld, "lbl", iHeight, iWidth, iLeft, iTop) +
                            "></" + strCtlPrefix + ":FldLabel>";
                break;
            case ASPNetFldCtlTypeE.sdFldTextBox:
                strRetVal = "<" + strCtlPrefix + ":FldTextBox" + 
                            BuildFieldControlBody(fld, "fld", iHeight, iWidth, iLeft, iTop) + 
                            "></" + strCtlPrefix + ":FldTextBox>";
                break;
            case ASPNetFldCtlTypeE.sdFldDropDownList:
                strRetVal = "<" + strCtlPrefix + ":FldDropDownList" + 
                            BuildFieldControlBody(fld, "fld", iHeight, iWidth, iLeft, iTop) + 
                            "></" + strCtlPrefix + ":FldDropDownList>";
                break;
            case ASPNetFldCtlTypeE.sdFldListBox:
                strRetVal = "<" + strCtlPrefix + ":FldListBox" + 
                            BuildFieldControlBody(fld, "fld", iHeight, iWidth, iLeft, iTop) + 
                            "></" + strCtlPrefix + ":FldListBox>";
                break;
            case ASPNetFldCtlTypeE.sdFldRadioButtonList:
                strRetVal = "<" + strCtlPrefix + ":FldRadioButtonList" + 
                            BuildFieldControlBody(fld, "fld", iHeight, iWidth, iLeft, iTop) + 
                            "></" + strCtlPrefix + ":FldRadioButtonList>";
                break;
            case ASPNetFldCtlTypeE.sdFldCheckBoxList:
                strRetVal = "<" + strCtlPrefix + ":FldCheckBoxList" + 
                            BuildFieldControlBody(fld, "fld", iHeight, iWidth, iLeft, iTop) + 
                            "></" + strCtlPrefix + ":FldCheckBoxList>";
                break;
            case ASPNetFldCtlTypeE.sdFldCalendar:
                strRetVal = "<" + strCtlPrefix + ":FldCalendar" + 
                            BuildFieldControlBody(fld, "fld", iHeight, iWidth, iLeft, iTop) + 
                            "></" + strCtlPrefix + ":FldCalender>";
                break;
            default:
                break;
            }
            return(strRetVal);
        }

        //********************************************************************************
        //
        /// <summary>
        /// GenASPNetFldCtlText:            Generate ASP.Net field control text
        /// </summary>
        /// <param name="fld">              Rep++ field</param>
        /// <param name="eCtlType">         ASP.Net control type</param>
        /// <param name="strCtlPrefix">     ASP.Net control prefix</param>
        /// <returns>
        /// ASP.Net control type
        /// </returns>
        //
        //********************************************************************************
        public static string GenASPNetFldCtlText(RepPP.Field          fld, 
                                                 ASPNetFldCtlTypeE    eCtlType,
                                                 string               strCtlPrefix)
        {
            return(GenASPNetFldCtlText(fld,
                                       eCtlType,
                                       strCtlPrefix,
                                       -1,
                                       -1,
                                       -1,
                                       -1));
        }

        //********************************************************************************
        //
        /// <summary>
        /// GenASPNetFldCtlText:            Generate ASP.Net field control text
        /// </summary>
        /// <param name="fld">              Rep++ field</param>
        /// <param name="eCtlType">         ASP.Net control type</param>
        /// <returns>
        /// ASP.Net control type
        /// </returns>
        //
        //********************************************************************************
        public static string GenASPNetFldCtlText(RepPP.Field          fld, 
                                                 ASPNetFldCtlTypeE    eCtlType) {
            return(GenASPNetFldCtlText(fld,
                                       eCtlType,
                                       DEF_CTL_PREFIX,
                                       -1,
                                       -1,
                                       -1,
                                       -1));
        }
	} // class ASPNetFieldGenerator
} // namespace
